@@ -54,75 +54,76 @@ void iplink_usage(void)
 	if (iplink_have_newlink()) {
 		fprintf(stderr,
 			"Usage: ip link add [link DEV] [ name ] NAME\n"
-			"                   [ txqueuelen PACKETS ]\n"
-			"                   [ address LLADDR ]\n"
-			"                   [ broadcast LLADDR ]\n"
-			"                   [ mtu MTU ] [index IDX ]\n"
-			"                   [ numtxqueues QUEUE_COUNT ]\n"
-			"                   [ numrxqueues QUEUE_COUNT ]\n"
-			"                   type TYPE [ ARGS ]\n"
+			"		    [ txqueuelen PACKETS ]\n"
+			"		    [ address LLADDR ]\n"
+			"		    [ broadcast LLADDR ]\n"
+			"		    [ mtu MTU ] [index IDX ]\n"
+			"		    [ numtxqueues QUEUE_COUNT ]\n"
+			"		    [ numrxqueues QUEUE_COUNT ]\n"
+			"		    type TYPE [ ARGS ]\n"
 			"\n"
-			"       ip link delete { DEVICE | dev DEVICE | group DEVGROUP } type TYPE [ ARGS ]\n"
+			"	ip link delete { DEVICE | dev DEVICE | group DEVGROUP } type TYPE [ ARGS ]\n"
 			"\n"
-			"       ip link set { DEVICE | dev DEVICE | group DEVGROUP }\n"
-			"	                  [ { up | down } ]\n"
-			"	                  [ type TYPE ARGS ]\n");
+			"	ip link set { DEVICE | dev DEVICE | group DEVGROUP }\n"
+			"			[ { up | down } ]\n"
+			"			[ type TYPE ARGS ]\n");
 	} else
 		fprintf(stderr,
 			"Usage: ip link set DEVICE [ { up | down } ]\n");
 
 	fprintf(stderr,
-		"	                  [ arp { on | off } ]\n"
-		"	                  [ dynamic { on | off } ]\n"
-		"	                  [ multicast { on | off } ]\n"
-		"	                  [ allmulticast { on | off } ]\n"
-		"	                  [ promisc { on | off } ]\n"
-		"	                  [ trailers { on | off } ]\n"
-		"	                  [ carrier { on | off } ]\n"
-		"	                  [ txqueuelen PACKETS ]\n"
-		"	                  [ name NEWNAME ]\n"
-		"	                  [ address LLADDR ]\n"
-		"	                  [ broadcast LLADDR ]\n"
-		"	                  [ mtu MTU ]\n"
-		"	                  [ netns { PID | NAME } ]\n"
-		"	                  [ link-netns NAME | link-netnsid ID ]\n"
-		"			  [ alias NAME ]\n"
-		"	                  [ vf NUM [ mac LLADDR ]\n"
-		"				   [ vlan VLANID [ qos VLAN-QOS ] [ proto VLAN-PROTO ] ]\n"
-		"				   [ rate TXRATE ]\n"
-		"				   [ max_tx_rate TXRATE ]\n"
-		"				   [ min_tx_rate TXRATE ]\n"
-		"				   [ spoofchk { on | off} ]\n"
-		"				   [ query_rss { on | off} ]\n"
-		"				   [ state { auto | enable | disable} ] ]\n"
-		"				   [ trust { on | off} ] ]\n"
-		"				   [ node_guid { eui64 } ]\n"
-		"				   [ port_guid { eui64 } ]\n"
-		"			  [ { xdp | xdpgeneric | xdpdrv | xdpoffload } { off |\n"
+		"		[ arp { on | off } ]\n"
+		"		[ dynamic { on | off } ]\n"
+		"		[ multicast { on | off } ]\n"
+		"		[ allmulticast { on | off } ]\n"
+		"		[ promisc { on | off } ]\n"
+		"		[ trailers { on | off } ]\n"
+		"		[ carrier { on | off } ]\n"
+		"		[ txqueuelen PACKETS ]\n"
+		"		[ name NEWNAME ]\n"
+		"		[ address LLADDR ]\n"
+		"		[ broadcast LLADDR ]\n"
+		"		[ mtu MTU ]\n"
+		"		[ netns { PID | NAME } ]\n"
+		"		[ link-netns NAME | link-netnsid ID ]\n"
+		"			[ alias NAME ]\n"
+		"			[ vf NUM [ mac LLADDR ]\n"
+		"				 [ vlan VLANID [ qos VLAN-QOS ] [ proto VLAN-PROTO ] ]\n"
+		"				 [ rate TXRATE ]\n"
+		"				 [ max_tx_rate TXRATE ]\n"
+		"				 [ min_tx_rate TXRATE ]\n"
+		"				 [ spoofchk { on | off} ]\n"
+		"				 [ query_rss { on | off} ]\n"
+		"				 [ state { auto | enable | disable} ] ]\n"
+		"				 [ trust { on | off} ] ]\n"
+		"				 [ node_guid { eui64 } ]\n"
+		"				 [ port_guid { eui64 } ]\n"
+		"			[ { xdp | xdpgeneric | xdpdrv | xdpoffload } { off |\n"
 		"				  object FILE [ section NAME ] [ verbose ] |\n"
 		"				  pinned FILE } ]\n"
-		"			  [ master DEVICE ][ vrf NAME ]\n"
-		"			  [ nomaster ]\n"
-		"			  [ addrgenmode { eui64 | none | stable_secret | random } ]\n"
-		"			  [ protodown { on | off } ]\n"
-		"			  [ gso_max_size BYTES ] | [ gso_max_segs PACKETS ]\n"
+		"			[ master DEVICE ][ vrf NAME ]\n"
+		"			[ nomaster ]\n"
+		"			[ addrgenmode { eui64 | none | stable_secret | random } ]\n"
+		"			[ protodown { on | off } ]\n"
+		"			[ gso_max_size BYTES ] | [ gso_max_segs PACKETS ]\n"
 		"\n"
-		"       ip link show [ DEVICE | group GROUP ] [up] [master DEV] [vrf NAME] [type TYPE]\n");
-
-	fprintf(stderr, "\n       ip link xstats type TYPE [ ARGS ]\n");
-	fprintf(stderr, "\n       ip link afstats [ dev DEVICE ]\n");
+		"	ip link show [ DEVICE | group GROUP ] [up] [master DEV] [vrf NAME] [type TYPE]\n"
+		"\n"
+		"	ip link xstats type TYPE [ ARGS ]\n"
+		"\n"
+		"	ip link afstats [ dev DEVICE ]\n");
 
 	if (iplink_have_newlink()) {
 		fprintf(stderr,
 			"\n"
-			"       ip link help [ TYPE ]\n"
+			"	ip link help [ TYPE ]\n"
 			"\n"
 			"TYPE := { vlan | veth | vcan | vxcan | dummy | ifb | macvlan | macvtap |\n"
-			"          bridge | bond | team | ipoib | ip6tnl | ipip | sit | vxlan |\n"
-			"          gre | gretap | erspan | ip6gre | ip6gretap | ip6erspan |\n"
-			"          vti | nlmon | team_slave | bond_slave | bridge_slave |\n"
-			"          ipvlan | ipvtap | geneve | vrf | macsec | netdevsim | rmnet |\n"
-			"          xfrm }\n");
+			"	   bridge | bond | team | ipoib | ip6tnl | ipip | sit | vxlan |\n"
+			"	   gre | gretap | erspan | ip6gre | ip6gretap | ip6erspan |\n"
+			"	   vti | nlmon | team_slave | bond_slave | bridge_slave |\n"
+			"	   ipvlan | ipvtap | geneve | vrf | macsec | netdevsim | rmnet |\n"
+			"	   xfrm }\n");
 	}
 	exit(-1);
 }
@@ -945,7 +946,8 @@ int iplink_parse(int argc, char **argv, struct iplink_req *req, char **type)
 	else if (!strcmp(name, dev))
 		name = dev;
 
-	if (dev && addr_len) {
+	if (dev && addr_len &&
+	    !(req->n.nlmsg_flags & NLM_F_CREATE)) {
 		int halen = nl_get_ll_addr_len(dev);
 
 		if (halen >= 0 && halen != addr_len) {
